/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.jpdl.internal.activity;

import java.util.List;
import java.util.Map;

import org.jbpm.api.activity.ActivityExecution;
import org.jbpm.api.env.Environment;
import org.jbpm.api.model.Activity;
import org.jbpm.api.model.Transition;
import org.jbpm.api.task.Task;
import org.jbpm.api.task.TaskHandler;
import org.jbpm.jpdl.internal.model.JpdlExecution;
import org.jbpm.pvm.internal.model.ExecutionImpl;
import org.jbpm.pvm.internal.session.DbSession;
import org.jbpm.pvm.internal.task.TaskDefinitionImpl;
import org.jbpm.pvm.internal.task.TaskImpl;


/**
 * @author Tom Baeyens
 */
public class TaskActivity extends JpdlExternalActivity {

  private static final long serialVersionUID = 1L;

  protected TaskDefinitionImpl taskDefinition;
  
  public void execute(ActivityExecution execution) {
    JpdlExecution jpdlExecution = execution.getExtension(JpdlExecution.class);
    TaskImpl task = jpdlExecution.createTask(taskDefinition);

    TaskHandler taskHandler = task.getTaskHandler();
    boolean wait = taskHandler.executionCreateTask(task);
    
    if (wait) {
      execution.waitForSignal();
    }
  }
  
  public void signal(ActivityExecution execution, String signalName, Map<String, ?> parameters) {
    Activity activity = execution.getActivity();
    
    if (parameters!=null) {
      execution.setVariables(parameters);
    }
    
    execution.fire(signalName, activity);

    DbSession taskDbSession = Environment
        .getFromCurrent(DbSession.class);
    TaskImpl task = (TaskImpl) taskDbSession.findTaskByExecution(execution);
    TaskHandler taskHandler = task.getTaskHandler();
    taskHandler.executionSignal(task);
    
    Transition transition = null;
    List<Transition> outgoingTransitions = activity.getOutgoingTransitions();
    if ( (outgoingTransitions!=null)
         && (!outgoingTransitions.isEmpty())
       ) {
      transition = activity.findOutgoingTransition(signalName);
      if (transition==null) {
        if (Task.STATE_COMPLETED.equals(signalName)) {
          if (outgoingTransitions.size()==1) {
            transition = outgoingTransitions.get(0);
          } else {
            transition = activity.getDefaultOutgoingTransition();
          }
        } else {
          // if a user specified outcome was provided and it doesn't
          // match with an outgoing transition name, then the process 
          // instance is suspended.  parked for admin intervention.
          ((ExecutionImpl)execution.getProcessInstance()).suspend();
        }
      }
      if (transition!=null) {
        execution.take(transition);
      }
    }
  }

  public TaskDefinitionImpl getTaskDefinition() {
    return taskDefinition;
  }
  public void setTaskDefinition(TaskDefinitionImpl taskDefinition) {
    this.taskDefinition = taskDefinition;
  }
}
