/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.pvm.internal.cmd;

import org.jbpm.api.JbpmException;
import org.jbpm.api.cmd.Command;
import org.jbpm.api.env.Environment;
import org.jbpm.api.model.Comment;
import org.jbpm.pvm.internal.model.CommentImpl;
import org.jbpm.pvm.internal.session.DbSession;


/**
 * @author Tom Baeyens
 */
public class AddReplyCommentCmd extends AbstractCommand<Comment> {

  private static final long serialVersionUID = 1L;
  
  protected long commentDbid;
  protected String message;
  
  public AddReplyCommentCmd(long commentDbid, String message) {
    this.commentDbid = commentDbid;
    this.message = message;
  }

  public Comment execute(Environment environment) throws Exception {
    DbSession dbSession = environment.get(DbSession.class);
    CommentImpl parentComment = dbSession.get(CommentImpl.class, commentDbid);
    if (parentComment==null) {
      throw new JbpmException("parent comment doesn't exist: "+commentDbid);
    }
    Comment replyComment = parentComment.createComment(message);
    return replyComment;
  }
}
