/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.pvm.internal.cmd;

import java.util.List;

import org.hibernate.Query;
import org.hibernate.Session;
import org.jbpm.api.JbpmException;
import org.jbpm.api.env.Environment;
import org.jbpm.api.task.Participation;
import org.jbpm.pvm.internal.task.ParticipationImpl;


/**
 * @author Tom Baeyens
 */
public class GetParticipantsCmd extends AbstractCommand<List<Participation>> {
  
  private static final long serialVersionUID = 1L;

  protected Long taskDbid;
  protected Long swimlaneDbid;
  
  public GetParticipantsCmd(Long taskDbid, Long swimlaneDbid) {
    this.taskDbid = taskDbid;
    this.swimlaneDbid = swimlaneDbid;
  }

  public List<Participation> execute(Environment environment) throws Exception {
    StringBuilder hql = new StringBuilder();
    hql.append("select role from ");
    hql.append(ParticipationImpl.class.getName());
    hql.append(" as role where ");
    
    if (taskDbid!=null) {
      hql.append(" role.task.dbid = :taskDbid ");

    } else if (swimlaneDbid!=null) {
      hql.append(" role.swimlane.dbid = :swimlaneDbid ");
      
    } else {
      throw new JbpmException("no task nor swimlane specified");
    }

    Session session = Environment.getFromCurrent(Session.class);
    Query query = session.createQuery(hql.toString());

    if (taskDbid!=null) {
      query.setLong("taskDbid", taskDbid);

    } else if (swimlaneDbid!=null) {
      query.setLong("swimlaneDbid", swimlaneDbid);
    } 

    List<Participation> participations = query.list();
    return participations;
  }
}
