/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.pvm.internal.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.jbpm.api.JbpmException;
import org.jbpm.api.env.Environment;
import org.jbpm.api.model.Comment;
import org.jbpm.pvm.internal.task.TaskImpl;
import org.jbpm.pvm.internal.util.Clock;
import org.jbpm.pvm.internal.util.EqualsUtil;

public class CommentImpl implements Serializable, Comment {

  private static final long serialVersionUID = 1L;

  protected long dbid;
  protected int dbversion;
  protected String userId = null;
  protected Date time = null;
  protected String message = null;
  protected List<CommentImpl> comments = null;

  public CommentImpl() {
  }

  public CommentImpl(String message) {
    this.message = message;
    this.time = Clock.getCurrentTime();
    
    Environment environment = Environment.getCurrent();
    if (environment!=null) {
      this.userId = environment.getUserId();
    }
  }
  
  // replies //////////////////////////////////////////////////////////////////
  
  public Comment createComment(String message) {
    if (message==null) {
      throw new JbpmException("message is null");
    }
    if (this.comments==null) {
      comments = new ArrayList<CommentImpl>();
    }
    CommentImpl reply = new CommentImpl(message);
    comments.add(reply);
    return reply;
  }

  public void removeComment(Comment comment) {
    if (comment==null) {
      throw new JbpmException("reply is null");
    }
    if (comments!=null) {
      comments.remove(comment);
    }
  }
  
  public List<Comment> getComments() {
    if (comments==null) {
      return Collections.EMPTY_LIST;
    }
    return (List)comments;
  }
  
  
  public void setComments(List<CommentImpl> replies) {
    this.comments = replies;
  }

  // equals ///////////////////////////////////////////////////////////////////
  // hack to support comparing hibernate proxies against the real objects
  // since this always falls back to ==, we don't need to overwrite the hashcode
  public boolean equals(Object o) {
    return EqualsUtil.equals(this, o);
  }

  // getters and setters //////////////////////////////////////////////////////

  public long getDbid() {
    return dbid;
  }
  public String getUserId() {
    return userId;
  }
  public String getMessage() {
    return message;
  }
  public Date getTime() {
    return time;
  }
  public void setUserId(String userId) {
    this.userId = userId;
  }
  public void setTime(Date time) {
    this.time = time;
  }
  public void setMessage(String message) {
    this.message = message;
  }
}
