/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.pvm.internal.svc;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jbpm.api.TaskQuery;
import org.jbpm.api.TaskService;
import org.jbpm.api.model.Comment;
import org.jbpm.api.task.*;
import org.jbpm.pvm.internal.cmd.*;
import org.jbpm.pvm.internal.task.TaskImpl;
import org.jbpm.pvm.internal.task.TaskQueryImpl;

/**
 * @author Alejandro Guizar, Tom Baeyens
 * @author Heiko Braun <heiko.braun@jboss.com>
 */
public class TaskServiceImpl extends AbstractServiceImpl implements TaskService {

  public Task newTask() {
    return commandService.execute(new NewTaskCmd(null));
  }

  public Task getTask(long taskDbid) {
    return commandService.execute(new GetTaskCmd(taskDbid));
  }

  public long saveTask(Task task) {
    return commandService.execute(new SaveTaskCmd((TaskImpl) task));
  }

  public void deleteTask(long taskDbid) {
    commandService.execute(new CancelTaskCmd(taskDbid));
  }

  public void deleteTask(long taskDbid, String reason) {
    commandService.execute(new CancelTaskCmd(taskDbid, reason));
  }

  public void completeTask(long taskDbid) {
    completeTask(taskDbid, null);
  }

  public void completeTask(long taskDbid, String outcome) {
    commandService.execute(new CompleteTaskCmd(taskDbid, outcome));
  }

  public void addTaskParticipatingUser(long taskDbid, String userId, String participation) {
    commandService.execute(new AddParticipationCmd(taskDbid, null, userId, null, participation));
  }

  public void addTaskParticipatingGroup(long taskDbid, String groupId, String participation) {
    commandService.execute(new AddParticipationCmd(taskDbid, null, null, groupId, participation));
  }

  public List<Participation> getTaskParticipations(long taskDbid) {
    return commandService.execute(new GetParticipantsCmd(taskDbid, null));
  }

  public void removeTaskParticipatingUser(long taskDbid, String userId, String participation) {
    commandService.execute(new RemoveParticipantCmd(taskDbid, null, userId, null, participation));
  }

  public void removeTaskParticipatingGroup(long taskDbid, String groupId, String participation) {
    commandService.execute(new RemoveParticipantCmd(taskDbid, null, null, groupId, participation));
  }


  public List<Task> findPersonalTasks(String userId) {
    return createTaskQuery()
        .assignee(userId)
        .orderDesc(TaskQuery.PROPERTY_PRIORITY)        
        .list();
  }

  public List<Task> findGroupTasks(String userId) {
    return createTaskQuery()
        .candidate(userId)
        .orderDesc(TaskQuery.PROPERTY_PRIORITY)        
        .list();
  }

  public TaskQuery createTaskQuery() {
    return new TaskQueryImpl(commandService);
  }

  public List<Task> getSubTasks(long taskDbid) {
    return commandService.execute(new GetSubTasksCmd(taskDbid));
  }

  public Task newTask(long parentTaskDbid) {
    return commandService.execute(new NewTaskCmd(parentTaskDbid));
  }

  public Comment addTaskComment(long taskDbid, String message) {
    return commandService.execute(new AddTaskCommentCmd(taskDbid, message));
  }

  public List<Comment> getTaskComments(long taskDbid) {
    return commandService.execute(new GetTaskCommentsCmd(taskDbid));
  }

  public void deleteComment(long commentDbid) {
    commandService.execute(new DeleteCommentCmd(commentDbid));
  }

  public Comment addReplyComment(long commentDbid, String message) {
    return commandService.execute(new AddReplyCommentCmd(commentDbid, message));
  }

  public void assignTask(long taskDbid, String userId) {
    commandService.execute(new AssignTaskCmd(taskDbid, userId));
  }

  public void takeTask(long taskDbid, String userId) {
    commandService.execute(new AssignTaskCmd(taskDbid, userId, true));
  }

  public Object getVariable(long taskDbid, String variableName) {
    Set<String> variableNames = new HashSet<String>();
    variableNames.add(variableName);
    GetVariablesCmd cmd = new GetVariablesCmd(taskDbid, variableNames);
    Map<String, Object> variables = commandService.execute(cmd);
    return variables.get(variableName);
  }

  public Set<String> getVariableNames(long taskDbid) {
    return commandService.execute(new GetVariableNamesCmd(taskDbid));
  }

  public Map<String, Object> getVariables(long taskDbid, Set<String> variableNames) {
    return commandService.execute(new GetVariablesCmd(taskDbid, variableNames));
  }

  public void setVariable(long taskDbid, String name, Object value) {
    SetVariablesCmd cmd = new SetVariablesCmd(taskDbid);
    cmd.addVariable(name, value);
    commandService.execute(cmd);
  }

  public void setVariables(long taskDbid, Map<String, Object> variables) {
    SetVariablesCmd cmd = new SetVariablesCmd(taskDbid);
    cmd.setVariables(variables);
    commandService.execute(cmd);
  }

  public Set<String> getOutcomes(long taskDbid) {
    return commandService.execute(new GetOutcomes(taskDbid));
  }
}
